<?php

namespace Drupal\ai_migrate_agent\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;

class AiMigrateAgentForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ai_migrate_agent_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['migration_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Migration Source File'),
      '#description' => $this->t('Upload a migration source file (e.g., CSV).'),
      '#required' => TRUE,
      '#upload_location' => 'public://migration_uploads/',
      '#upload_validators' => [
        'file_validate_extensions' => ['csv'],
      ],
    ];
    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Start Migration'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $file_fid = $form_state->getValue('migration_file')[0] ?? NULL;
    if ($file_fid && ($file = \Drupal::entityTypeManager()->getStorage('file')->load($file_fid))) {
      // Set file as permanent.
      $file->setPermanent();
      $file->save();
      $filename = $file->getFilename();
      $pathinfo = pathinfo($filename);
      $basename = $pathinfo['basename'];
      $filepath = $file->getFileUri();
      $file_contents = file_get_contents($file->getFileUri());

      // Infer source type from file extension.
      $extension = pathinfo($filename, PATHINFO_EXTENSION);
      $source_type = match (strtolower($extension)) {
        'csv' => 'csv',
        'json' => 'json',
        'xml' => 'xml',
        default => 'unknown',
      };

      $this->messenger()->addStatus($this->t('TODO: Send the following to the AI'));
      $this->messenger()->addStatus($this->t('<blockquote>Create a Migration Group about %basename if one does not exist, with source type %type. Within that Migration Group, create a Migration Entity with the source type of "%type" and the file path of "%filepath".</blockquote>', [
        '%file' => $filename,
        '%type' => $source_type,
        '%filepath' => $filepath,
        '%basename' => $basename,
      ]));
    } else {
      $this->messenger()->addError($this->t('File upload failed.'));
    }
  }

}
