<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use League\HTMLToMarkdown\HtmlConverter;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\migrate_plus\Entity\MigrationGroup;

/**
 * Plugin implementation of the field validation results.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:create_migration_entity',
  function_name: 'ai_migrate_agent_create_migration_entity',
  name: 'Create Migration Entity',
  description: 'This function will create a Migration entity, which is part of a Migration Group, and tells the Migration which source and destination to use. After the Migration Entity is created, you will need to configure the Migration Source and Destination.',
  group: 'migrate_tools',
  context_definitions: [
    'name' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Name"),
      description: new TranslatableMarkup("The name of the Migrate Entity"),
      required: TRUE
    ),
    'migration_tags' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Migration Tags"),
      description: new TranslatableMarkup("The tags to associate with the Migration"),
      required: FALSE,
      multiple: TRUE
    ),
    // 'source' => new ContextDefinition(
    //   data_type: 'string',
    //   label: new TranslatableMarkup("Source"),
    //   description: new TranslatableMarkup("The source type plugin to use when migrating content, like csv or json."),
    //   required: TRUE
    // ),
    // 'destination' => new ContextDefinition(
    //   data_type: 'string',
    //   label: new TranslatableMarkup("Destination"),
    //   description: new TranslatableMarkup("The destination where the content will be migrated into, like Nodes, Taxonomy Terms, or Users."),
    //   required: TRUE
    // ),
  ]
)]
class CreateMigrationEntity extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * Load from dependency injection container.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    $name = $this->getContextValue('name');
    $migration_tags = $this->getContextValue('migration_tags') ?? [];
    // $source = $this->getContextValue('source');
    // $destination = $this->getContextValue('destination');

    // Prepare the migration entity values.
    $migration_values = [
      'id' => strtolower(preg_replace('/[^a-zA-Z0-9_]+/', '_', $name)),
      'label' => $name,
      'migration_tags' => $migration_tags,
      // 'source' => [
      //   'plugin' => $source,
      // ],
      // 'destination' => [
      //   'plugin' => $destination,
      // ],
      'migration_group' => 'default',
    ];

    // Create the migration entity.
    /** @var \Drupal\migrate_plus\Entity\Migration $migration */
    $migration = \Drupal\migrate_plus\Entity\Migration::create($migration_values);
    $migration->save();

    $this->setOutput('The Migrate Entity "' . $migration->label() . '" has been created with ID "' . $migration->id() . '".');
  }

}
