<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin to execute a migration (import or rollback).
 */
#[FunctionCall(
  id: 'ai_migrate_agent:execute_migration',
  function_name: 'ai_migrate_agent_execute_migration',
  name: 'Execute Migration',
  description: 'Executes a Migration import or rollback with the Migrate module.',
  group: 'migrate_tools',
  context_definitions: [
    'migration_id' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Migration ID"),
      description: new TranslatableMarkup("The migration entity to execute."),
      required: TRUE
    ),
    'operation' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Operation"),
      description: new TranslatableMarkup("The operation to perform: import or rollback."),
      required: TRUE
    ),
  ]
)]
class ExecuteMigration extends FunctionCallBase implements ExecutableFunctionCallInterface {
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  public function execute() {
    $migration_id = $this->getContextValue('migration_id');
    $operation = $this->getContextValue('operation');
    $migration = \Drupal\migrate_plus\Entity\Migration::load($migration_id);
    if ($migration) {
      $executable = \Drupal::service('plugin.manager.migration')->createInstance($migration_id);
      if ($operation === 'import') {
        $result = $executable->import();
        $this->setOutput('Migration import executed for "' . $migration_id . '". Result: ' . print_r($result, TRUE));
      } elseif ($operation === 'rollback') {
        $result = $executable->rollback();
        $this->setOutput('Migration rollback executed for "' . $migration_id . '". Result: ' . print_r($result, TRUE));
      } else {
        $this->setOutput('Unknown operation: ' . $operation);
      }
    } else {
      $this->setOutput('Migration entity not found: ' . $migration_id);
    }
  }
}
