<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin to get migration status.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:get_migration_status',
  function_name: 'ai_migrate_agent_get_migration_status',
  name: 'Migration Status',
  description: 'Retrieves the status of a migration.',
  group: 'migrate_tools',
  context_definitions: [
    'migration_id' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Migration ID"),
      description: new TranslatableMarkup("The migration entity to check status for."),
      required: TRUE
    ),
  ]
)]
class GetMigrationStatus extends FunctionCallBase implements ExecutableFunctionCallInterface {
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  public function execute() {
    $migration_id = $this->getContextValue('migration_id');
    $migration = \Drupal\migrate_plus\Entity\Migration::load($migration_id);
    if ($migration) {
      $status = $migration->get('status');
      $this->setOutput('Migration status for "' . $migration_id . '": ' . $status);
    } else {
      $this->setOutput('Migration entity not found: ' . $migration_id);
    }
  }
}
