<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use League\HTMLToMarkdown\HtmlConverter;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\migrate_plus\Entity\MigrationGroup;

/**
 * Plugin implementation of the field validation results.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:list_migrate_source_plugins',
  function_name: 'ai_migrate_agent_list_migrate_source_plugins',
  name: 'List Migrate Source Types',
  description: 'This retrieves all available Migrate Source Types.',
  group: 'migrate_tools'
)]
class ListMigrateSourcePlugins extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * Load from dependency injection container.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    // Get the migration source plugin manager.
    $plugin_manager = \Drupal::service('plugin.manager.migrate.source');

    if (!isset($plugin_manager)) {
      $this->setOutput('Failed to get the migrate source plugins.');
    }

    // Get all plugin definitions.
    $plugin_definitions = $plugin_manager->getDefinitions();

    if (empty($plugin_definitions)) {
      $this->setOutput('There are currently no Migrate plugin sources.');
      return;
    }

    // Construct the output.
    $output = [];
    foreach ($plugin_definitions as $id => $plugin) {
      array_push($output, '- ' . $id);
    }

    $this->setOutput(implode(PHP_EOL, $output));
  }

}
