<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Plugin to list all available migration destination plugins.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:list_migration_destination_plugins',
  function_name: 'ai_migrate_agent_list_migration_destination_plugins',
  name: 'List Migration Destination Plugins',
  description: 'Lists all available migration destination plugins that source data can be imported into.',
  group: 'migrate_tools'
)]
class ListMigrationDestinationPlugins extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  public function execute() {
    $definitions = \Drupal::service('plugin.manager.migrate.destination')->getDefinitions();
    if (empty($definitions)) {
      $this->setOutput('No migration destination plugins found.');
      return;
    }
    $output = [];
    foreach ($definitions as $id => $definition) {
      array_push($output, $id);
    }
    $yaml_output = Yaml::dump($output, 2, 2);
    $this->setOutput($yaml_output);
  }
}
