<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

use Drupal\migrate_plus\Entity\Migration;

/**
 * Plugin implementation to list available Migration Entities.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:list_migration_entities',
  function_name: 'ai_migrate_agent_list_migration_entities',
  name: 'List Migration Entities',
  description: 'This retrieves all available Migration Entities with details.',
  group: 'migrate_tools'
)]
class ListMigrationEntities extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * Load from dependency injection container.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    $migrations = Migration::loadMultiple();

    if (empty($migrations)) {
      $this->setOutput('There are currently no Migration Entities defined.');
      return;
    }

    $output = [];
    foreach ($migrations as $migration) {
      /** @var \Drupal\migrate_plus\Entity\Migration $migration */
      $output[$migration->id()] = [
        'label' => $migration->label(),
        'migration_group' => $migration->get('migration_group'),
        'migration_tags' => $migration->get('migration_tags'),
        'source' => $migration->get('source'),
        'destination' => $migration->get('destination'),
        'status' => $migration->get('status'),
        'dependencies' => $migration->get('dependencies'),
      ];
    }

    // Output as YAML.
    $yaml_output = Yaml::dump($output, 2, 2);
    $this->setOutput($yaml_output);
  }

}
