<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

use Drupal\migrate_plus\Entity\MigrationGroup;

/**
 * Plugin implementation to list available Migrate Groups.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:list_migration_groups',
  function_name: 'ai_migrate_agent_list_migration_groups',
  name: 'List Migration Groups',
  description: 'This retrieves all available Migration Groups.',
  group: 'migrate_tools'
)]
class ListMigrationGroups extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * Load from dependency injection container.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    $groups = MigrationGroup::loadMultiple();

    if (empty($groups)) {
      $this->setOutput('There are currently no Migrate Groups defined.');
      return;
    }

    $output = [];
    foreach ($groups as $group) {
      /** @var \Drupal\migrate_plus\Entity\MigrationGroup $group */
      $output[$group->id()] = [
        'label' => $group->label(),
        'description' => $group->get('description'),
        'source_type' => $group->get('source_type'),
      ];
    }

    // Output as YAML.
    $yaml_output = Yaml::dump($output, 2, 2);
    $this->setOutput($yaml_output);
  }

}
